package com.eyefodder.flexcover.agents
{
	import com.allurent.coverage.Controller;
	import com.allurent.coverage.ControllerFactory;
	import com.allurent.coverage.model.CoverageModelFactory;
	import com.allurent.coverage.model.application.IRecorder;
	import com.allurent.coverage.model.application.RecorderFactory;
	import com.allurent.coverage.runtime.ICoverageAgent;
	import com.allurent.coverage.service.CoverageCommunicatorFactory;
	import com.eyefodder.flexcover.model.CommandLineOptionsModel;
	import com.eyefodder.flexcover.parse.CommandLineOptionsParser;
	
	import flash.desktop.NativeApplication;
	import flash.events.InvokeEvent;
	import flash.utils.getQualifiedClassName;
	
	import mx.logging.ILogger;
	import mx.logging.Log;

	public class HeadlessCoverageAgent implements ICoverageAgent
	{
		private var log:ILogger;
		private var controller:Controller;
		private var recorder:IRecorder;
		private var optionsModel:CommandLineOptionsModel;
		private var initialised:Boolean = false;
		
		public function HeadlessCoverageAgent()
		{
			log = Log.getLogger(getQualifiedClassName(this).replace("::","."));
			controller = Controller(ControllerFactory.instance);
			NativeApplication.nativeApplication.addEventListener(InvokeEvent.INVOKE, onAppInvoked);
			recorder = RecorderFactory.instance;
			recorder.autoUpdate = false;
		}


		private function onAppInvoked(evt:InvokeEvent):void{
			log.info("headless agent invoked");
			NativeApplication.nativeApplication.removeEventListener(InvokeEvent.INVOKE, onAppInvoked);
			var parser:CommandLineOptionsParser = new CommandLineOptionsParser();
			optionsModel = parser.parse(evt.arguments);
			var optionsErrors:Array = optionsModel.getErrorMessagesForHeadless();
			if(optionsErrors.length==0){
				setup(optionsModel);	
				initialised = true;			
			}else{
				log.error( "Missing enough command line args to initiate headless debugger");
				for each (var errorMessage:String in optionsErrors){
					
					log.debug(errorMessage);
				}
			}
		
		//	parseCommandLineArgs(evt.arguments);
		}
		
		private function setup(vo:CommandLineOptionsModel):void{
            controller.setup(CoverageModelFactory.instance, 
            	RecorderFactory.instance, 
            	CoverageCommunicatorFactory.instance);	
            
            controller.coverageOutputFile = vo.coverageReportCVRFormat;
            controller.emmaOutputFile = vo.coverageReportEMMAFormat;
            
            
            Controller(controller).loadMetadata(vo.metadataFile);
		}
		
		
		private function enoughCommandLineArgs(vo:CommandLineOptionsModel):Boolean{
			if(vo.metadataFile == null){
				return false;
			}
			
			return true;
		}

		public function recordCoverage(key:String):void
		{
			/** @PBH - in the local Connection Agent there is a lot of caching and saving here, but in the headless agent this shouldnt be necessary
			 */
			 if(initialised){
				 var objectMap:Object = {}
				 objectMap[key] = 1;
				 recorder.record(  objectMap );
			 }
		}
		
		public function exit():void
		{
			if(initialised){
				//applys coverage data in one pass
				//recorder.applyCoverageData();
				//write coverage data to file:
				//writeCoverageData();
				
				controller.close();
			}

		}
		
		
		
	}
}